@@ -14,6 +14,12 @@
 #' correspond to the seven plot \code{option}s defined by their last letter,
 #' and return the data for plotting, e.g. for use with \code{ggplot2}.
 #'
+#' The \code{trim} option allows unsightly long line segments to be omitted
+#' from plots with options 'a' or 'u'. It ranks the line segments on the basis
+#' of the age gap (dx) and the distance of the midpoint of the line from the
+#' mean curve (dy) using the formula \code{abs(dx)/mad(dx) + abs(dy)/mad(dy)}
+#' and omits those with the largest values.
+
 #' @aliases plot.sitar lines.sitar plot_d plot_v plot_D plot_V plot_u
 #'  plot_a plot_c
 #' @param x object of class \code{sitar}.
@@ -38,7 +44,8 @@
 #' calculated as the age when the second derivative of the fitted curve changes
 #' sign (after applying \code{xfun} and/or \code{yfun}). Age at peak velocity
 #' is marked in the plot with a vertical dotted line, and its value, along with
-#' peak velocity, is printed and returned.
+#' peak velocity, is printed and returned. NB their standard errors can be
+#' obtained using the bootstrap with the function \code{apv_se}.
 #' @param xfun optional function to be applied to the x variable prior to
 #' plotting. Defaults to NULL, which translates to \code{ifun(x$call.sitar$x)}
 #' and inverts any transformation applied to x in the original SITAR model
@@ -58,6 +65,8 @@
 #' values are used. If \code{abc} is set, \code{level} is ignored. If
 #' \code{abc} is NULL (default), or if a, b or c values are missing, values of
 #' zero are assumed.
+#' @param trim number (default 0) of long line segments to be excluded from plot
+#' with option 'u' or 'a'. See Details.
 #' @param add optional logical defining if the plot is pre-existing (TRUE) or
 #' new (FALSE). TRUE is equivalent to using \code{lines}.
 #' @param nlme optional logical which set TRUE plots the model as an
@@ -80,7 +89,7 @@
 #' \item{usr2}{the value of \code{par('usr')} for the velocity (y2) plot.}
 #' \item{apv}{if argument \code{apv} is TRUE a named list giving the age at
 #' peak velocity (apv) and peak velocity (pv) from the fitted velocity curve,
-#' either overall or (with options D or V) for all subjects.}
+#' either overall or (with options D or V, invisibly) for all subjects.}
 #' If \code{returndata} is TRUE (which it is with the helper functions) returns
 #' invisibly either a tibble or named list of tibbles,
 #' containing the data to be plotted. The helper functions each return a tibble.
@@ -90,7 +99,7 @@
 #' in the original \code{sitar} call then '.x' corresponds by default to \code{age}.
 #' @author Tim Cole \email{tim.cole@@ucl.ac.uk}
 #' @seealso \code{\link{mplot}},
-#' \code{\link{plotclean}}, \code{\link{ifun}}
+#' \code{\link{plotclean}}, \code{\link{ifun}}, \code{\link{apv_se}}
 #' @keywords aplot
 #' @examples
 #'
@@ -99,9 +108,14 @@
 #'
 #' ## draw fitted distance and velocity curves
 #' ## with velocity curve in blue
-#' ## adding age at peak velocity
+#' ## adding age at peak velocity (apv)
 #' plot(m1, y2par=list(col='blue'), apv=TRUE)
 #'
+#' ## bootstrap standard errors for apv and pv
+#' \dontrun{
+#' res <- apv_se(m1, nboot=20, plot=TRUE)
+#' }
+
 #' ## draw individually coloured growth curves adjusted for random effects
 #' ## using same x-axis limits as for previous plot
 #' plot(m1, opt='a', col=id, xlim=xaxsd())
@@ -122,11 +136,13 @@
 
 #' @importFrom grDevices xy.coords
 #' @importFrom graphics plot axis identify legend lines locator par text title mtext abline
-#' @importFrom tibble tibble
-#' @importFrom dplyr mutate
+#' @importFrom tibble tibble as_tibble
+#' @importFrom dplyr mutate rename filter
+#' @importFrom rlang .data as_label
+#' @importFrom glue glue
 #' @export
 plot.sitar <- function(x, opt="dv", labels, apv=FALSE, xfun=NULL, yfun=NULL, subset=NULL,
-                       ns=101, abc=NULL, add=FALSE, nlme=FALSE,
+                       ns=101, abc=NULL, trim=0, add=FALSE, nlme=FALSE,
                        returndata=FALSE, ...,
                        xlab=NULL, ylab=NULL, vlab=NULL,
                        xlim=c(NA, NA), ylim=c(NA, NA), vlim=c(NA, NA),
@@ -150,7 +166,8 @@ plot.sitar <- function(x, opt="dv", labels, apv=FALSE, xfun=NULL, yfun=NULL, sub
       lapply(as.list(dots), eval, data, parent.frame())
     ARG1 <- ARG[names(ARG) != 'y2par']
     ARG2 <- ARG[['y2par']]
-    if ('las' %in% names(ARG1)) ARG2$las <- ARG1$las
+    if ('las' %in% names(ARG1))
+      ARG2$las <- ARG1$las
 
 #	plot x & y1 axes
     do.call('plot', c(list(x=xlim, y=ylim, type='n', xlab=xlab, ylab=ylab), ARG1))
@@ -161,7 +178,8 @@ plot.sitar <- function(x, opt="dv", labels, apv=FALSE, xfun=NULL, yfun=NULL, sub
     if (dv == 3) {
       par(new=TRUE)
       plot(xlim, vlim, type='n', bty='n', ann=FALSE, axes=FALSE)
-      do.call('axis', list(side=4))
+      localaxis <- function(..., col, bg, pch, cex, lty, lwd) axis(...)
+      do.call('localaxis', c(list(side=4), ARG2))
       mtext(vlab, 4, par('mgp')[1])
 #	save y2 axis limits
       xy$usr2 <- par('usr')
@@ -187,280 +205,315 @@ plot.sitar <- function(x, opt="dv", labels, apv=FALSE, xfun=NULL, yfun=NULL, sub
     seq(rx[1], rx[2], length.out=n)
   }
 
-  distance <- velocity <- function(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, n=ns) {
+  distance <- velocity <- function(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, ns=ns) {
 # generate x values across the range to plot mean spline curve
-    dvt <- rlang::quo_name(match.call()[[1]])
+    dvt <- as_label(match.call()[[1]])
     dvt <- as.numeric(dvt == 'velocity')
     .x <- getCovariate(model)[subset]
-    .x <- xseq(.x, n)
-    . <- tibble::tibble(
-      .y=predict(model, tibble::tibble(.x), level=0, deriv=dvt, abc=abc, xfun=xfun, yfun=yfun),
-      .x=xfun(.x)
+    .x <- xseq(.x, ns)
+    newdata <- tibble(.x)
+    if (sum(subset) < length(subset)) attr(newdata, 'subset') <- subset
+    level <- ifelse(is.null(abc), 0, 1)
+    . <- tibble(
+      .x=xfun(.x),
+      .y=predict(model, newdata, level=level, deriv=dvt, abc=abc, xfun=xfun, yfun=yfun)
     )
-    if (length(.x[subset]) != length(.x)) attr(., 'subset') <- subset
-    .[, c('.x', '.y')]
   }
 
-  Distance <- Velocity <- function(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, n=ns) {
+  Distance <- Velocity <- function(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, ns=ns) {
 # generate x and id values across the range to plot spline curves
-    dvt <- rlang::quo_name(match.call()[[1]])
+    dvt <- as_label(match.call()[[1]])
     dvt <- as.numeric(dvt == 'Velocity')
     .x <- getCovariate(model)[subset]
     .id <- getGroups(model)[subset]
-    npt <- n / diff(range(.x))
-    . <- by(tibble::tibble(.x, .id), .id, function(z) {
-      xrange <- range(z$.x)
-      nt <- floor(npt * diff(xrange)) + 1
-      tibble::tibble(
-        .x=xseq(xrange, nt),
-        .id=z$.id[[1]]
+    npt <- ns / diff(range(.x))
+    if (is.null(abc)) {
+      . <- by(tibble(.x, .id), .id, function(z) {
+        xrange <- range(z$.x)
+        nt <- ceiling(npt * diff(xrange))
+        tibble(
+          .x=xseq(xrange, nt),
+          .id=z$.id[[1]]
+        )
+      })
+      . <- do.call('rbind', .)
+    } else {
+      . <- tibble(
+        .x=xseq(.x, ns),
+        .id=.id[[1]],
       )
-    })
-    . <- do.call('rbind', .)
-    . <- dplyr::mutate(.,
+    }
+    . <- mutate(.,
                 .y=predict(model, ., deriv=dvt, abc=abc, xfun=xfun, yfun=yfun),
                 .x=xfun(.x)
     )
     .[, c('.x', '.y', '.id')]
   }
 
-  unadjusted <- function(model, subset=subset, xfun=xfun, yfun=yfun) {
+  unadjusted <- function(model, subset=subset, xfun=xfun, yfun=yfun, trim=trim) {
 # unadjusted individual curves
-    tibble::tibble(
-      .x=xfun(getCovariate(model)[subset]),
-      .y=yfun(getResponse(model)[subset]),
-      .id=getGroups(model)[subset]
-    )
+    data <- tibble(
+      .x=getCovariate(model),
+      .y=getResponse(model),
+      .id=getGroups(model)) %>%
+      filter(subset)
+    data <- trimlines(model, data, level=1, trim) %>%
+      mutate(.x=xfun(.data$.x),
+             .y=yfun(.data$.y))
+    data
   }
 
-  adjusted <- function(model, subset=subset, xfun=xfun, yfun=yfun) {
+  adjusted <- function(model, subset=subset, xfun=xfun, yfun=yfun, trim=trim) {
 # adjusted individual curves
-    . <- xyadj(model)
-    tibble::tibble(
-      .x=xfun(.$x)[subset],
-      .y=yfun(.$y)[subset],
-      .id=getGroups(model)[subset]
-    )
+    data <- as_tibble(xyadj(model)) %>%
+      mutate(.id=getGroups(model)) %>%
+      rename(.x=.data$x,
+             .y=.data$y) %>%
+      filter(subset)
+    data <- trimlines(model, data, level=0, trim) %>%
+      mutate(.x=xfun(.data$.x),
+             .y=yfun(.data$.y))
+    data
   }
 
-  crosssectional <- function(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, n=ns) {
+  trimlines <- function(model, data, level, trim) {
+# if midpoint of line segment is far from mean curve, or age gap is large,
+# insert NA row to data to omit line segment
+    if (trim == 0)
+      return(data)
+    data <- with(data, data[order(.id, .x), ]) # sort data
+    extra <- as_tibble(diff(as.matrix(data[, 1:2]))) # diff for .x and .y
+    extra$.id <- data$.id[-1] # save .id
+    did <- diff(as.integer(data$.id)) # 1+ = change of .id, 0 = .id
+    extra$dx <- extra$.x # save dx age gap
+    extra[, 1:2] <- data[-1, 1:2] - extra[, 1:2] / 2 # midpoints for .x and .y
+    extra <- extra[!did, ] # restrict to same .id
+    extra$ey <- predict(model, extra, level=level) # predicted y value at midpoint
+    extra <- extra %>%
+      mutate(dy=abs(.data$.y - .data$ey), # gap between line segment and mean curve dy
+             xy=.data$dx / mad(.data$dx) + .data$dy / mad(.data$dy)) # add scaled dx and dy
+    outliers <- order(extra$xy, decreasing=TRUE)[1:trim] # identify outliers
+    extra <- extra[outliers, 1:3] # trim
+    extra$.y <- NA
+    data <- rbind(data, extra)
+    with(data, data[order(.id, .x), ]) # sort data
+  }
+
+  crosssectional <- function(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, ns=ns) {
 # fixed effect mean curve
     x <- getCovariate(model)[subset]
-    x <- xseq(x, n) - model$xoffset
-    . <- tibble::tibble(
-      .y=yfun(predict(model$ns, tibble::tibble(x))),
+    x <- xseq(x, ns) - model$xoffset
+    . <- tibble(
+      .y=yfun(predict(model$ns, tibble(x))),
       .x=xfun(x + model$xoffset)
     )
     .[, c('.x', '.y')]
   }
 
-  dolegend <- function(ypar, y2par, legend) {
+  dolegend <- function(ARG1, ARG2, legend) {
 # add legend
     parlu <- function(...) par(do.call('par', list(...)))
+    if (is.null(ARG2$lty))
+      ARG2$lty <- 2
     llc <- sapply(c('lty', 'lwd', 'col'), function(i) {
       p12 <- rep(par()[[i]], 2)
-      if (!is.null(ypar[[i]]))
-        p12[1] <- parlu(ypar[i])[[1]]
-      if (!is.null(y2par[[i]]))
-        p12[2] <- parlu(y2par[i])[[1]]
+      for (j in 1:2) {
+        k <- list(ARG1[i], ARG2[i])[[j]]
+        if (!is.null(k[[1]]) && length(k[[1]]) == 1)
+          p12[j] <- parlu(k)[[1]]
+      }
       p12
     })
     do.call('legend', c(legend, list(lty=llc[, 'lty'], lwd=llc[, 'lwd'], col=llc[, 'col'])))
   }
 
-  if (nlme) {
+  if (nlme)
     do.call('plot.lme', as.list(match.call()[-1]))
-  }
-  else {
-    model <- x
-    data <- getData(model)
-    mcall <- model$call.sitar
+  model <- x
+  data <- getData(model)
+# check data versus model
+  if (nrow(data) != model$dims$N)
+    stop(glue('lengths of data ({nrow(data)}) and model ({model$dims$N}) do not match'))
+  mcall <- model$call.sitar
 #	extract list(...)
-    ccall <- match.call(expand.dots=FALSE)
+  ccall <- match.call(expand.dots=FALSE)
 #	subset to plot model
-    subset <- eval(ccall$subset, data, parent.frame())
-    if (is.null(subset))
-      subset <- rep_len(TRUE, model$dims$N)
+  subset <- eval(ccall$subset, data, parent.frame())
+  if (is.null(subset))
+    subset <- rep_len(TRUE, model$dims$N)
 # ... args
-    dots <- ccall$...
-    ARG <- if (!is.null(dots))
-      lapply(as.list(dots), eval, data[subset, ], parent.frame())
-    ARG1 <- ARG[names(ARG) != 'y2par']
-    ARG2 <- ARG[['y2par']]
+  dots <- ccall$...
+  ARG <- if (!is.null(dots))
+    lapply(as.list(dots), eval, data[subset, ], parent.frame())
 
-    options   <- c('d', 'c', 'u', 'a', 'D', 'v', 'V')
-    optnames  <- c('distance', 'crosssectional', 'unadjusted', 'adjusted', 'Distance', 'velocity', 'Velocity')
-    optaxis   <- c( 1,   1,   1,   1,   1,   2,   2 ) # default y1=1, y2=2
-    optmult   <- c( FALSE, FALSE, TRUE,  TRUE,  TRUE,  FALSE, TRUE ) # multiple curves
-    optsmooth <- c( TRUE,  TRUE,  FALSE, FALSE, TRUE,  TRUE,  TRUE ) # spline curves
-    optDV     <- c( FALSE, FALSE, FALSE, FALSE, TRUE,  FALSE, TRUE ) # extended curves
-    opts <- unique(na.omit(match(unlist(strsplit(opt, '')), options)))
-    if (length(opts) == 0)
-      stop('option(s) not recognised')
-    dv <- range(optaxis[opts])
-    dv <- min(dv) + diff(dv) * 2 # 1=d, 2=v, 3=dv
+  options   <- c('d', 'c', 'u', 'a', 'D', 'v', 'V')
+  optnames  <- c('distance', 'crosssectional', 'unadjusted', 'adjusted', 'Distance', 'velocity', 'Velocity')
+  optaxis   <- c( 1,   1,   1,   1,   1,   2,   2 ) # default y1=1, y2=2
+  optmult   <- c( FALSE, FALSE, TRUE,  TRUE,  TRUE,  FALSE, TRUE ) # multiple curves
+  optsmooth <- c( TRUE,  TRUE,  FALSE, FALSE, TRUE,  TRUE,  TRUE ) # spline curves
+  opts <- unique(na.omit(match(unlist(strsplit(opt, '')), options)))
+  if (length(opts) == 0)
+    stop('option(s) not recognised')
+  dv <- range(optaxis[opts])
+  dv <- min(dv) + diff(dv) * 2 # 1=d, 2=v, 3=dv
 
 # create missing labels
-    if (missing(labels))
-      labels <- vector('character', 3)
-    else if (length(labels) < 3)
-      labels <- c(labels, '', '')
-    if (labels[3] == '' && labels[2] != '')
-      labels[3] <- paste(labels[2], 'velocity')
+  if (missing(labels))
+    labels <- vector('character', 3)
+  else if (length(labels) < 3)
+    labels <- c(labels, '', '')
+  if (labels[3] == '' && labels[2] != '')
+    labels[3] <- paste(labels[2], 'velocity')
 
 #	if xlab not specified replace with label or x name (depending on xfun)
-    if (is.null(xlab)) {
-      if (labels[1] == '') {
-        xlab <- if (!is.null(xfun))
-          paste0('(', deparse(substitute(xfun)), ')(', deparse(mcall$x), ")")
-        else
-          attr(ifun(mcall$x), 'varname')
-        labels[1] <- xlab
-      }
+  if (is.null(xlab)) {
+    if (labels[1] == '') {
+      xlab <- if (!is.null(xfun))
+        paste0('(', deparse(substitute(xfun)), ')(', deparse(mcall$x), ")")
       else
-        xlab <- labels[1]
+        attr(ifun(mcall$x), 'varname')
+      labels[1] <- xlab
     }
+    else
+      xlab <- labels[1]
+  }
 
 #	if ylab not specified replace with label or y name (depending on yfun)
-    if (is.null(ylab)) {
-      if (labels[2] == '') {
-        ylab <- if (!is.null(yfun))
-          paste0('(', deparse(substitute(yfun)), ')(', deparse(mcall$y), ")")
-        else
-          attr(ifun(mcall$y), 'varname')
-        labels[2] <- ylab
-      }
+  if (is.null(ylab)) {
+    if (labels[2] == '') {
+      ylab <- if (!is.null(yfun))
+        paste0('(', deparse(substitute(yfun)), ')(', deparse(mcall$y), ")")
       else
-        ylab <- labels[2]
+        attr(ifun(mcall$y), 'varname')
+      labels[2] <- ylab
     }
+    else
+      ylab <- labels[2]
+  }
 
 #	if vlab not specified replace with label or v name
-    if (is.null(vlab)) {
-      if (labels[3] == '')
-        labels[3] <- paste(labels[2], 'velocity')
-      vlab <- labels[3]
-    }
+  if (is.null(vlab)) {
+    if (labels[3] == '')
+      labels[3] <- paste(labels[2], 'velocity')
+    vlab <- labels[3]
+  }
 
 # derive xfun and yfun
-    if (is.null(xfun))
-      xfun <- ifun(mcall$x)
-    if (is.null(yfun))
-      yfun <- ifun(mcall$y)
+  if (is.null(xfun))
+    xfun <- ifun(mcall$x)
+  if (is.null(yfun))
+    yfun <- ifun(mcall$y)
 
 # generate list of data frames for selected options
-    data <- lapply(opts, function(i) {
-      if (optsmooth[[i]])
-        do.call(optnames[[i]], list(model=model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, n=ns))
-      else
-        do.call(optnames[[i]], list(model=model, subset=subset, xfun=xfun, yfun=yfun))
-    })
+  data <- lapply(opts, function(i) {
+    if (optsmooth[[i]])
+      do.call(optnames[[i]], list(model=model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, ns=ns))
+    else
+      do.call(optnames[[i]], list(model=model, subset=subset, xfun=xfun, yfun=yfun, trim=trim))
+  })
 
 # return data?
-    if (returndata){
-      if (length(data) == 1)
-        data <- data[[1]]
-      else
-        names(data) <- options[opts]
-      return(invisible(data))
-    }
+  if (returndata){
+    if (length(data) == 1)
+      data <- data[[1]]
+    else
+      names(data) <- options[opts]
+    return(invisible(data))
+  }
 
 # extract axis ranges and plot axes
-    if (!add) {
-      if (any(is.na(xlim)))
-        xlim <- range(vapply(data, function(z) range(z$.x), 0:1/2))
-      if (any(is.na(ylim)) && any(optaxis[opts] == 1))
-        ylim <- range(vapply(data[optaxis[opts] == 1], function(z) range(z$.y), 0:1/2))
-      if (any(is.na(vlim)) && any(optaxis[opts] == 2))
-        vlim <- range(vapply(data[optaxis[opts] == 2], function(z) range(z$.y), 0:1/2))
-      xy <- do.call('plotaxes',
-                    c(list(dv=dv, xlab=xlab, ylab=ylab, vlab=vlab,
-                            xlim=xlim, ylim=ylim, vlim=vlim), ARG))
+  if (!add) {
+    if (any(is.na(xlim)))
+      xlim <- range(vapply(data, function(z) range(z$.x, na.rm=TRUE), numeric(2)))
+    if (any(is.na(ylim)) && any(optaxis[opts] == 1))
+      ylim <- range(vapply(data[optaxis[opts] == 1], function(z) range(z$.y, na.rm=TRUE), numeric(2)))
+    if (any(is.na(vlim)) && any(optaxis[opts] == 2))
+      vlim <- range(vapply(data[optaxis[opts] == 2], function(z) range(z$.y, na.rm=TRUE), numeric(2)))
+    xy <- do.call('plotaxes',
+                  c(list(dv=dv, xlab=xlab, ylab=ylab, vlab=vlab,
+                          xlim=xlim, ylim=ylim, vlim=vlim), ARG))
 # add legend
-      if (dv == 3) {
-        # default dotted line for velocity curve
-        if (is.null(ARG$y2par$lty)) {
-          ARG$y2par$lty <- 2
-          ARG2 <- ARG$y2par
-        }
-        if (!is.null(legend)) {
-          legend[['legend']] <- labels[2:3]
-          dolegend(ARG1, ARG2, legend)
-        }
-      }
+    if (dv == 3 && !is.null(legend)) {
+      legend[['legend']] <- labels[2:3]
+      dolegend(ARG[names(ARG) != 'y2par'], ARG$y2par, legend)
     }
+  }
 # else retrieve axis ranges
-    else {
-      xy <- list()
-      xy$usr <- par('usr')
-      xlim <- xaxsd()
-      ylim <- yaxsd()
-      vlim <- .par.usr2
-      if (!is.null(vlim)) {
-        xy$usr2 <- vlim
-        vlim <- yaxsd(.par.usr2[3:4])
-        dv <- 3
-      } else if (dv == 3)
-          stop('right y axis not set up')
-    }
+  else {
+    xy <- list()
+    xy$usr <- par('usr')
+    xlim <- xaxsd()
+    ylim <- yaxsd()
+    if (exists('.par.usr2') && !is.null(.par.usr2)) {
+      dv <- 3
+      vlim <- yaxsd(.par.usr2[3:4])
+      xy$usr2 <- .par.usr2
+    } else if (dv == 3)
+        stop('right y axis not set up')
+  }
 
 # plot curves
-    lapply(1:length(opts), function(i) {
-      opt <- opts[[i]]
-      . <- data[[i]]
-      ARG0 <- ARG
-      # if D or V and dots, extend ARG
-      if (optDV[opt] && !is.null(dots)) {
-        names(.) <- unlist(as.list(mcall[2:(length(.)+1)]))
-        ARG0 <- lapply(as.list(dots), eval, ., parent.frame())
-      }
-      # select distance or velocity axis
-      if (optaxis[opt] == 1 || dv < 3) {
-        fun <- I
-        ARG0 <- ARG0[names(ARG0) != 'y2par']
-      } else {
-        fun <- v2d(ylim, vlim)
-        ARG0 <- ARG0[['y2par']]
-      }
-      if (optmult[opt])
-        do.call("mplot", c(list(x=.[[1]], y=fun(.[[2]]), id=.[[3]], add=TRUE), ARG0))
-      else
-        xy <- do.call("lines", c(list(x=.[[1]], y=fun(.[[2]])), ARG0))
-    })
+  lapply(1:length(opts), function(i) {
+    opt <- opts[[i]]
+    . <- data[[i]]
+    ARG0 <- ARG
+# data frame extended, extend ARG
+    if (optmult[opt] && nrow(.) != model$dims$N && !is.null(dots)) {
+      names(.) <- unlist(as.list(mcall[2:(length(.) + 1)]))
+      ARG0 <- lapply(as.list(dots), eval, ., parent.frame())
+    }
+# select distance or velocity axis
+    if (optaxis[opt] == 1 || dv < 3) {
+      fun <- I
+      ARG0 <- ARG0[names(ARG0) != 'y2par']
+    } else {
+      fun <- v2d(ylim, vlim)
+      ARG0 <- ARG0[['y2par']]
+      if (is.null(ARG0[['lty']]))
+        ARG0[['lty']] <- 2
+    }
+    if (optmult[opt])
+      do.call("mplot", c(list(x=.[[1]], y=fun(.[[2]]), id=.[[3]], add=TRUE), ARG0))
+    else
+      xy <- do.call("lines", c(list(x=.[[1]], y=fun(.[[2]])), ARG0))
+  })
 
 # save and print vertical line(s) at age of peak velocity
-    if (apv) {
+  if (apv) {
 # single curve
-        xy$apv <- with(velocity(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, n=ns),
-                       setNames(getPeakTrough(.x, .y), c('apv', 'pv')))
-      print(signif(xy$apv, 4))
-      if (any(optmult[opts])) {
+      xy$apv <- with(velocity(model, subset=subset, abc=abc, xfun=xfun, yfun=yfun, ns=ns),
+                     setNames(getPeakTrough(.x, .y), c('apv', 'pv')))
+    print(signif(xy$apv, 4))
+    if (any(optmult[opts])) {
 # multiple curves
-        if (body(ifun(mcall$x)) == as.name('x') &&
-            body(ifun(mcall$y)) == as.name('x')) {
+      ids <- levels(factor(getGroups(model)[subset]))
+      if (body(xfun) == as.name('x') &&
+          body(yfun) == as.name('x')) {
 # x and y untransformed
-          apv1 <- xyadj(model, xy$apv[1], tomean=FALSE)$x # subject-specific APVs
-          pv1 <- xy$apv[2] * exp(ranef(model)$c) # subject-specific PVs
-          xy$apv <- data.frame(apv=apv1, pv=pv1)
-        }
+        apv1 <- xyadj(model, xy$apv[1], y=0, id=ids, tomean=FALSE)$x # subject-specific APVs
+        pv1 <- xy$apv[2] * exp(ranef(model)$c) # subject-specific PVs
+        xy$apv <- data.frame(apv=apv1, pv=pv1)
+      }
 # xfun or yfun set
-        else {
-          xt <- xseq(getCovariate(model)[subset])
-          newdata <- data.frame(.x=xt)
-          . <- vapply(levels(factor(getGroups(model)[subset])), function(z) {
-            newdata$.id <- z
-            vt <- predict(object=model, newdata=newdata, deriv=1, abc=abc, xfun=xfun, yfun=yfun)
-            getPeakTrough(xfun(xt), vt)
-          }, c(0, 0))
-          xy$apv <- setNames(data.frame(t(.)), c('apv', 'pv'))
-        }
+      else {
+        xt <- xseq(getCovariate(model)[subset])
+        newdata <- data.frame(.x=xt)
+        if (!is.null(abc))
+          ids <- factor(1, labels=ids[[1]])
+        . <- vapply(ids, function(z) {
+          newdata$.id <- z
+          vt <- predict(object=model, newdata=newdata, deriv=1, abc=abc, xfun=xfun, yfun=yfun)
+          getPeakTrough(xfun(xt), vt)
+        }, numeric(2))
+        xy$apv <- setNames(data.frame(t(.)), c('apv', 'pv'))
       }
-# plot apv
-      ARG2$lty <- 3
-      do.call('abline', c(list(v=unlist(xy$apv['apv'])), ARG2))
     }
-# return xy
-    invisible(xy)
+# plot apv
+    do.call('abline', list(v=unlist(xy$apv['apv']), lty=3))
   }
+# return xy
+  invisible(xy)
+
 }
 #############################
 #
